{% extends 'admin/change_form.html' %}

{% block submit_buttons_bottom %}
    {{ block.super }}
    <div class="submit-row">
            {% csrf_token %}
                <input type="submit" value="Test SMTP account" name="_test_smtp">
    </div>
{% endblock %}



